///
/// The configuration module is responsible for parsing both the Kinesis Agent compatible
/// configuration and other formats as well
///
use serde::{Deserialize, Serialize};

#[derive(Serialize, Deserialize, Debug)]
pub struct KinesisConfig {
    #[serde(rename = "checkpointFile")]
    checkpoint_file: String,
    #[serde(rename = "kinesis.endpoint")]
    endpoint: String,
    #[serde(default)]
    flows: Vec<KinesisFlow>,
}

#[derive(Serialize, Deserialize, Debug)]
struct KinesisFlow {
    #[serde(rename = "filePattern")]
    file_pattern: String,
    #[serde(rename = "initialPosition")]
    initial_position: FilePosition,
    #[serde(rename = "kinesisStream")]
    kinesis_stream: String,
}

#[derive(Serialize, Deserialize, Debug, PartialEq)]
enum FilePosition {
    #[serde(rename = "START_OF_FILE")]
    StartOfFile,
    #[serde(rename = "END_OF_FILE")]
    EndOfFile,
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn it_deserializes_no_flows() {
        let buf = r#"
{
    "checkpointFile" : "/tmp/foo.log",
    "kinesis.endpoint" : "someendpoint:9092"
}"#;
        let des: KinesisConfig = serde_json::from_str(&buf).unwrap();
        assert_eq!(des.checkpoint_file, "/tmp/foo.log");
        assert_eq!(des.endpoint, "someendpoint:9092");
    }

    #[test]
    fn it_deserializes_complete() {
        let buf = r#"
{
  "checkpointFile": "/tmp/aws-kinesis-agent-checkpoints/main.log",
  "kinesis.endpoint": "https://kinesis.us-west-2.amazonaws.com",
  "flows": [
    {
      "filePattern": "/tmp/test1.log*",
      "initialPosition": "END_OF_FILE",
      "kinesisStream": "test1"
    },
    {
      "filePattern": "/tmp/test2.log*",
      "initialPosition": "START_OF_FILE",
      "kinesisStream": "test2"
    }
  ]
}"#;
        let des: KinesisConfig = serde_json::from_str(&buf).unwrap();
        assert!(des.flows.len() == 2 "Expecting to deserialize two flows");

        assert!(
            des.flows[0].kinesis_stream == "test1",
            "Expecting the stream to equal `test1`, not {}",
            des.flows[0].kinesis_stream
        );
        assert!(des.flows[1].initial_position == FilePosition::StartOfFile);
    }
}
